/* -*- mode:prolog; -*-
 ******************************************************************
 * $Id$
 * Copyright (C) 2008 by INRIA 
 * Author: Eric de la Clergerie <Eric.De_La_Clergerie@inria.fr>
 * ----------------------------------------------------------------
 *
 *  fsa2dyalog.pl -- compilation FSA vers clauses DyALog
 *
 * ----------------------------------------------------------------
 * Description
 * 
 * Utilisation
 *    ./fsa2dyalog numbers.fsa > numbers_fsa.pl
 *    dyacc numbers_fsa.pl -o numbers_fsa
 *    ./numbers_fsa -- numbers cent deux
 *    ./numbers_fsa -- numbers -gen numbers
 *
 * Pour compiler  partir d'une regexp,
 * composer regexp2fsa avec fsa2dyalog
 *    ./regexp2fsa numbers.pl | ./fsa2dyalog - > numbers_1.pl
 *    dyacc numbers_1.pl -o numbers_1
 *
 * ----------------------------------------------------------------
 */

%% Import de la bibliothque DyALog pour l'impression
:-require 'format.pl'.

%% Les prdicat base de donnes pour les FSA
:-extensional init_state/2.
:-extensional final_state/2.
:-extensional trans/4.

%% Pour ne pas tabuler le prdicat fsa2dyalog
%% et avoir un comportement standard a la Prolog
:-rec_prolog fsa2dyalog/0.

fsa2dyalog :-
	init_state(Name,Init),
	%% emission de requetes pour chaque FSA
	emit_clause( ( ?- argv([Name|L]),
		          fsa_follow(Name,Init,L,[])
		     )
		    ),
	emit_clause( ( ?- argv(['-gen',Name]),
		       fsa_follow(Name,Init,L,[])
		     )
		   )
	.

fsa2dyalog :-
	final_state(Name,Final),
	emit_clause((fsa_follow(Name,Final,L,L))).

fsa2dyalog :-
	trans(Name,S,T,true),
	emit_clause((fsa_follow(Name,S,L,R) :- fsa_follow(Name,T,L,R))).

fsa2dyalog :-
	trans(Name,S,T,c(X)),
	%% on assume un lecteur liste (mais facile  adapter)
	emit_clause((fsa_follow(Name,S,[X|L],R) :- fsa_follow(Name,T,L,R))).

fsa2dyalog :-
	trans(Name,S,T,range(D)),
	%% on assume un lecteur liste (mais facile  adapter)
	%% et on genere une clause pour chaque valeur dans D
	domain(X,D),
	emit_clause((fsa_follow(Name,S,[X|L],R) :- fsa_follow(Name,T,L,R))).

:-std_prolog emit_clause/1.

emit_clause(Clause) :-
	format('~q.\n',[Clause]).

%% La requte chapeau pour la compilation
?-
  %% mission d'un petit header
  format('%% Compilation de FSA\n\n',[]),
  fsa2dyalog,
  fail.
