/* -*- mode:prolog; -*-
 ******************************************************************
 * $Id$
 * Copyright (C) 2008 by INRIA 
 * Author: Eric de la Clergerie <Eric.De_La_Clergerie@inria.fr>
 * ----------------------------------------------------------------
 *
 *  fst2dyalog.pl -- compilation FST vers clauses DyALog
 *
 * ----------------------------------------------------------------
 * Description
 *
 * Utilisation
 *    ./fst2dyalog numbers.fsa > numbers_fst.pl
 *    dyacc numbers_fst.pl -o numbers_fst
 *    ./numbers_fst -- numbers cent deux
 *    ./numbers_fst -- numbers -gen numbers
 *
 * Pour compiler  partir d'une regexp 2bandes,
 * composer regexp2fst avec fst2dyalog
 *    ./regexp2fst numbers.pl | ./fsa2dyalog - > numbers_2.pl
 *    dyacc numbers_2.pl -o numbers_2
 *
 * ----------------------------------------------------------------
 */

%% Import de la bibliothque DyALog pour l'impression
:-require 'format.pl'.

%% Les prdicat base de donnes pour les FSA
:-extensional init_state/2.
:-extensional final_state/2.
:-extensional trans/4.

%% Pour ne pas tabuler le prdicat fst2dyalog
%% et avoir un comportement standard a la Prolog
:-rec_prolog fst2dyalog/0.

fst2dyalog :-
	init_state(Name,Init),
	%% emission de requtes pour chaque FSA
	emit_clause( ( ?- argv([Name|L1]),
		          follow(Name,Init,L1,[],L2,[])
		     )
		    ),
	emit_clause( ( ?- argv(['-gen',Name]),
		       follow(Name,Init,L1,[],L2,[])
		     )
		   )
	.

fst2dyalog :-
	final_state(Name,Final),
	emit_clause((follow(Name,Final,L1,L1,L2,L2))).

fst2dyalog :-
	trans(Name,S,T,A1:A2),
	%% on assume un lecteur liste (mais facile  adapter)
	( A1 = true ->
	  L1 = M1
	; A1 = c(X1) ->
	  L1 = [X1|M1]
	; A1 = range(D1),
	  domain(X1,D1),
	  L1 = [X1|M1]
	),
	( A2 = true ->
	  L2 = M2
	; A2 = c(X2) ->
	  L2 = [X2|M2]
	; A2 = range(D2),
	  domain(X2,D2),
	  L2 = [X1|M2]
	),
	emit_clause((follow(Name,S,L1,R1,L2,R2) :- follow(Name,T,M1,R1,M2,R2))).

fst2dyalog :-
	trans(Name,S,T,srange(D)),
	domain(X1:X2,D),
	L1=[X1|M1],
	L2=[X2|M2],
	emit_clause((follow(Name,S,L1,R1,L2,R2) :- follow(Name,T,M1,R1,M2,R2))).

:-std_prolog emit_clause/1.

emit_clause(Clause) :-
	format('~q.\n',[Clause]).

%% La requte chapeau pour la compilation
?-
  %% mission d'un petit header
  format('%% Compilation de FST\n\n',[]),
  fst2dyalog,
  fail.
