/* -*- mode:prolog; -*-
 ******************************************************************
 * $Id$
 * Copyright (C) 2008 by INRIA 
 * Author: Eric de la Clergerie <Eric.De_La_Clergerie@inria.fr>
 * ----------------------------------------------------------------
 *
 *  regexp2.pl -- moteur pour les expressions rgulires 2 bandes
 *
 * ----------------------------------------------------------------
 * Description
 * 
 * ----------------------------------------------------------------
 */

%% Import de la bibliothque DyALog pour l'impression
:-require 'format.pl'.

%% Dclaration des oprateurs pour construire les regexp
:-op(yf,['@?'],500).
:-op(yf,['@+'],500).
:-op(yf,['@*'],500).


%% Destructurer les constructeurs de regexp

%% Concatnation
regexp2((E1,E2),L1,R1,L2,R2) :-
	regexp2(E1,L1,M1,L2,M2),
	regexp2(E2,M1,R1,M2,R2).

%% Disjunction
regexp2((E1;E2),L1,R1,L2,R2) :-
	( regexp2(E1,L1,R1,L2,R2)
	;
	  regexp2(E2,L1,R1,L2,R2)
	)
	.

%% toile de Kleene
regexp2(E @*,L1,R1,L2,R2) :-
	( L1=R1, L2=R2 ;
	  regexp2(E,L1,M1,L2,M2),
	  regexp2(E @*,M1,R1,M2,R2)
	).

%% toile de Kleene stricte
regexp2(E @+,L1,R1,L2,R2) :-
	regexp2(E,L1,M1,L2,M2),
	regexp2(E @*,M1,R1,M2,R2).

regexp2(E @?,L1,R1,L2,R2) :- regexp2((true;E),L1,R1,L2,R2).

%% Deux formes pour epsilon
regexp2(true,L1,L1,L2,L2).
regexp2([],L1,L1,L2,L2).

%% Les expressions de base
regexp2(A1:A2,L1,R1,L2,R2) :-
	regexp_action(A1,L1,R1),
	regexp_action(A2,L2,R2)
	.

regexp_action(true,L,L).
regexp_action(c(C),L,R) :- 'C'(L,C,R).
regexp_action(range(Range),[C|R],R) :- domain(C,Range).

%% Une action synchronise
regexp2(srange(D),L1,R1,L2,R2) :-
	'C'(L1,X1,R1),
	'C'(L2,X2,R2),
	domain(X1:X2,D)
	.

%% Rien n'empche d'ajouter des expressions de debug
regexp2(debug(Msg),L1,L1,L2,L2) :-
	format('~w at ~w and ~w\n',[Msg,L1,L2]).


%% Le lecteur sur liste (en complment du lecteur sur treillis de mots)
:-extensional 'C'/3.
'C'([C|R],C,R).

%% Le prdicat base de donnes pour accder aux regexp
:-extensional rx2/2.

%% Une requte possible
%% Lecture sur une liste prolog de symboles
?- argv([Name|L1]),
   rx2(Name,RX),
   regexp2(RX,L1,[],L2,[]).

%% Inverser les bandes
?-argv(['-rev',Name|SL2]),
  rx2(Name,RX),
  atoms2nums(SL2,L2),
  regexp2(RX,L1,[],L2,[])
  .

%% Tout generer
?-argv(['-gen',Name]),
  rx2(Name,RX),
  regexp2(RX,L1,[],L2,[])
  .

:-rec_prolog atoms2nums/2.

atoms2nums([],[]).
atoms2nums([Smb|SL],[Num|L]) :-
	atom_number(Smb,Num),
	atoms2nums(SL,L).





