/* -*- mode:prolog; -*-
 ******************************************************************
 * $Id$
 * Copyright (C) 2008 by INRIA 
 * Author: Eric de la Clergerie <Eric.De_La_Clergerie@inria.fr>
 * ----------------------------------------------------------------
 *
 *  regexp3.pl -- moteur pour les expressions rgulires 3 bandes
 *
 * ----------------------------------------------------------------
 * Description
 * 
 * ----------------------------------------------------------------
 */

%% Import de la bibliothque DyALog pour l'impression
:-require 'format.pl'.

%% Dclaration des oprateurs pour construire les regexp
:-op(yf,['@?'],500).
:-op(yf,['@+'],500).
:-op(yf,['@*'],500).


%% Destructurer les constructeurs de regexp

%% Concatnation
regexp3((E1,E2),L1,R1,L2,R2,L3,R3) :-
	regexp3(E1,L1,M1,L2,M2,L3,M3),
	regexp3(E2,M1,R1,M2,R2,M3,R3).

%% Disjunction
regexp3((E1;E2),L1,R1,L2,R2,L3,R3) :-
	( regexp3(E1,L1,R1,L2,R2,L3,R3)
	;
	  regexp3(E2,L1,R1,L2,R2,L3,R3)
	)
	.

%% toile de Kleene
regexp3(E @*,L1,R1,L2,R2,L3,R3) :-
	( L1=R1, L2=R2, L3=R3 ;
	  regexp3(E,L1,M1,L2,M2,L3,M3),
	  regexp3(E @*,M1,R1,M2,R2,M3,R3)
	).

%% toile de Kleene stricte
regexp3(E @+,L1,R1,L2,R2,L3,R3) :-
	regexp3(E,L1,M1,L2,M2,L3,M3),
	regexp3(E @*,M1,R1,M2,R2,M3,R3).

regexp3(E @?,L1,R1,L2,R2,L3,R3) :- regexp3((true;E),L1,R1,L2,R2,L3,R3).

%% Deux formes pour epsilon
regexp3(true,L1,L1,L2,L2,L3,L3).
regexp3([],L1,L1,L2,L2,L3,L3).

%% Les expressions de base
regexp3(A1:A2:A3,L1,R1,L2,R2,L3,R3) :-
	regexp_action(A1,L1,R1),
	regexp_action(A2,L2,R2),
	regexp_action(A3,L3,R3)
	.

regexp_action(true,L,L).
regexp_action(c(C),L,R) :- 'C'(L,C,R).
regexp_action(range(Range),[C|R],R) :- domain(C,Range).

%% Une action synchronise
regexp3(srange(D),L1,R1,L2,R2,L3,R3) :-
	'C'(L1,X1,R1),
	'C'(L2,X2,R2),
	'C'(L3,X3,R3),
	domain(X1:X2:X3,D)
	.

%% Rien n'empche d'ajouter des expressions de debug
regexp3(debug(Msg),L1,L1,L2,L2,L3,L3) :-
	format('~w at ~w and ~w and ~w\n',[Msg,L1,L2,L3]).


%% Le lecteur sur liste (en complment du lecteur sur treillis de mots)
:-extensional 'C'/3.
'C'([C|R],C,R).

%% Le prdicat base de donnes pour accder aux regexp
:-extensional rx3/2.

%% Une requte possible
%% Lecture sur une liste prolog de symboles
?- argv([Name|L1]),
   rx3(Name,RX),
   regexp3(RX,L1,[],L2,[],L3,[]).

%% Bande 2
?-argv(['-2',Name|SL2]),
  rx3(Name,RX),
  atoms2nums(SL2,L2),
  regexp3(RX,L1,[],L2,[],L3,[])
  .

%% Bande 3
?-argv(['-3',Name|L3]),
  rx3(Name,RX),
  regexp3(RX,L1,[],L2,[],L3,[])
  .

%% Tout generer
?-argv(['-gen',Name]),
  rx3(Name,RX),
  regexp3(RX,L1,[],L2,[],L3,[])
  .

:-rec_prolog atoms2nums/2.

atoms2nums([],[]).
atoms2nums([Smb|SL],[Num|L]) :-
	atom_number(Smb,Num),
	atoms2nums(SL,L).





